'
' Programm sendet ber die serielle Schnittstelle empfangene Zeichen als Morse aus.
' Geschwindigkeit und Tonhoehe sind ber Potentiometer an ADC1 und ADC0 einstellbar

' Pin PB3 wird als Timer2-Ausgang OC2 verwendet, Ton wird durch Umschalten des
' Pins als Eingang / Ausgang ein- und ausgeschaltet
'
' Die Morsezeichen sind hier nicht als Variablen- Array definiert, sondern werden
' per Lookup direkt in den Data nachgeschlagen
'
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
'Grundeinstelllungen:
$regfile = "m8def.dat"
$crystal = 3686400
$baud = 9600
'
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
'Variablen und Konstanten dimensionieren:
'

Dim I As Byte                                               'Fr temporren Gebrauch als Schleifenzhler
Dim Recbyte As Byte
Dim M As Byte                                               'Index des aktuellen Morsebuchstabens in unserer Tabelle
Dim L As Byte                                               ' fr niederwertigstes Bit beim bitweisen Erzeugen des Morsezeichens

Dim Tonhoehe As Word                                        'Tonhhe
Dim Punktlaenge As Word                                     'Punktlnge
Dim Strichlaenge As Word                                    'Strichlnge
Dim Wortpause As Word                                       'Lnge Wortpause

'
'
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Initialisierung des Programms:
'
Config Adc = Single , Prescaler = 16 , Reference = Avcc
Start Adc
'
' Tonsignal an OC2 / Port B 3 erzeugen; Signal wird unten durch Umschalten des DDRB im Takt der Zeichen geschaltet
'
Config Timer2 = Timer , Compare = Toggle , Prescale = 64 , Clear Timer = 1
Ocr2 = Tonhoehe

Acsr.acd = 0                                                'Analog-Komparator ausschalten, spart etwas Strom
Ddrb.3 = 0

Portb = &B11111111                                          'floatende Eingnge vermeiden
Portc = &B11111100
Portd = &B11111111

Config Serialin = Buffered , Size = 250                     ' Buffer fr serielle Schnittstelle einrichten
Enable Interrupts

'
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'


Main:

 Do

   Gosub Zeichen_lesen

 Loop


' Zeichen aus dem Schnittstellenpuffer abholen

Zeichen_lesen:

    If Ischarwaiting() = 1 Then                             ' wenn was da ist
       Recbyte = Waitkey()                                  ' einlesen
       Print Chr(recbyte);                                  ' auf Terminal Echo ausgeben
       M = Lookup(recbyte , Morsetabelle)                   ' Morsezeichen aus Tabelle holen
       Gosub Potis                                          ' Geschwindigkeit / Tonhhe
       Gosub Morse
    End If                                                  ' Morsezeichen erzeugen

Return


   ' Morsezeichenerzeugung:

  Morse:

      If M = 0 Then                                         ' Sonderfall: Leerzeichen;
         Waitms Wortpause
      Goto Zeichenende
      End If

      For I = 1 To 8
        If M = 1 Then                                       ' Das Byte hat nur noch den Wert 1; Zeichenende!
          Goto Zeichenende
        End If
        L = M And &B00000001                                ' niederwertigstes Bit lesen
          Ddrb.3 = 1                                        ' Ausgnge einschalten
          Portb.1 = 1
        If L = 1 Then
           Waitms Strichlaenge                              ' ist das Bit 1 -> dah
          Else
           Waitms Punktlaenge                               ' ist das Bit 0 -> dit
        End If
         Ddrb.3 = 0                                         ' Ausgnge abschalten
         Portb.1 = 0
        Waitms Punktlaenge                                  ' Pause innerhalb des Morsezeichens
        Shift M , Right , 1                                 ' Bits um eine Stelle nach rechts shiften
      Next I
     Zeichenende:
     Waitms Strichlaenge                                    ' Pause zwischen Morsezeichen
  Return



  Potis:

       Tonhoehe = Getadc(0)                                 ' Tonhoehe durch Poti einstellbar
       Tonhoehe = Tonhoehe + 768                            'fr sinnvollen Einstellbereich
       Tonhoehe = Tonhoehe / 24
       Ocr2 = Tonhoehe

       Punktlaenge = Getadc(1)                              ' Punktlnge durch Poti einstellbar;
       Punktlaenge = Punktlaenge + 512                      ' fr sinnvollen Einstellbereich
       Punktlaenge = Punktlaenge / 16                       ' aus der Punktlnge ergibt sich die Morsegeschwindigkeit
       Strichlaenge = Punktlaenge * 3                       ' Strichlnge: 3 Punktlngen
       Wortpause = Punktlaenge * 5                          ' Pause zwischen Worten: 5 Punktlngen

  Return


End


'
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'



Morsetabelle:



'Ascii 0 - 32 ; hier wird nur Linefeed in <kn> umgesetzt

Data 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , &B00101101 , 0 , , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0
Data 0 , 0 , 0 , 0 , 0 , 0 , 0

' Ascii 32 - 47   ->   _!"#$%&'()*+,-./
Data 0 , 0 , &B01010010 , 0 , 0 , 0 , &B00101010 , 0 , &B00101101 , &B01101101 , 0 , &B00101010 , &B01110011 , &B01100001
Data &B01101010 , &B00101001

'Ascii 48 - 57  ->  0123456789
Data &B00111111 , &B00111110 , &B00111100 , &B00111000 , &B00110000 , &B00100000 , &B00100001 , &B00100011 , &B00100111 , &B00101111

'Ascii 58 - 64  ->  :;<=>?@
Data &B01000111 , &B01110011 , &B00101101 , &B00110001 , &B01101101 , &B01001100 , 0

'Ascii 65 -  90        @ABCDEFGHIJKLMNOPQRSTUVWXYZ
Data &B00000110 , &B00010001 , &B00010101 , &B00001001 , &B00000010 , &B00010100 , &B00001011 , &B00010000 , &B00000100
Data &B00011110 , &B00001101 , &B00010010 , &B00000111 , &B00000101 , &B00001111 , &B00010110 , &B00011011 , &B00001010
Data &B00001000 , &B00000011 , &B00001100 , &B00011000 , &B00001110 , &B00011001 , &B00011101 , &B00010011

'Ascii 91 - 96  ->  Muell

Data 0 , 0 , 0 , 0 , 0 , 0

' Ascii 97 - 122  abcdrfghijklmnopqrstuvwxyz

Data &B00000110 , &B00010001 , &B00010101 , &B00001001 , &B00000010 , &B00010100 , &B00001011 , &B00010000 , &B00000100
Data &B00011110 , &B00001101 , &B00010010 , &B00000111 , &B00000101 , &B00001111 , &B00010110 , &B00011011 , &B00001010
Data &B00001000 , &B00000011 , &B00001100 , &B00011000 , &B00001110 , &B00011001 , &B00011101 , &B00010011

'Ascii 123 - 127  -> Muell